(**************************************************************)
(*
 *  Ensemble, (Version 0.61)
 *  Copyright 1999 Cornell University
 *  All rights reserved.
 *
 *  See ensemble/doc/license.txt for further information.
 *)
(**************************************************************)
(**************************************************************)
(* RESOURCE.ML *)
(* Author: Mark Hayden, 4/96 *)
(**************************************************************)
open Trans
open Util
(**************************************************************)
let name = Trace.file "RESOURCE"
let failwith = Trace.make_failwith name
(**************************************************************)

type ('a,'b) t = {
  name 		: name ;
  first 	: 'a -> 'b -> unit ;
  last 		: 'a -> 'b -> unit ;
  change	: ('a,'b) t -> unit ;
  mutable list 	: ('a * 'b) list ;
  mutable list2	: 'b list ;
  table 	: ('a,(int ref * 'b * debug)) Hashtbl.t
}

(**************************************************************)

let create name first last change = {
  name 	= name ;
  first = first ;
  last 	= last ;
  table = Hashtbl.create (*name*) 10 ;
  list 	= [] ;
  list2 = [] ;
  change = change
}

(**************************************************************)

let simple name =
  create name (fun _ _ -> ()) (fun _ _ -> ()) (fun _ -> ())

(**************************************************************)

let update e =
  e.list <- [] ;
  e.list2 <- [] ;
  Hashtbl.iter (fun k (_,d,_) ->
    e.list <- (k,d) :: e.list ;
    e.list2 <- d :: e.list2
  ) e.table ;
  e.change e

(**************************************************************)

let add e debug k d =
  try
    let (ctr,_,_) = Hashtbl.find e.table k in
    incr ctr
  with Not_found -> (
    e.first k d ;
    Hashtbl.add e.table k (ref 1,d,debug) ;
    update e
  )

(**************************************************************)

let remove e v =
  try
    let (ctr,d,_) = Hashtbl.find e.table v in
    decr ctr ;
    if !ctr = 0 then (
      e.last v d ;
      Hashtbl.remove e.table v ;
      update e
    )
  with Not_found ->
    failwith "remove:no such item in table"

(**************************************************************)

let to_list r = r.list
let to_list2 r = r.list2

(**************************************************************)

let info r =
  sprintf "RESOURCE:name=%s:size=%d" 
    r.name (hashtbl_size r.table)

let to_string r =
  let table = hashtbl_to_list r.table in
  string_of_list (fun (_,(count,_,debug)) -> sprintf "(%s,%d)" debug !count) table

(**************************************************************)
